/**
  *  \file server/queryout.cc
  */

#include "server/queryout.h"
#include "server/jsonout.h"
#include "game/fcode.h"
#include "game/global.h"
#include "cpluslib/format.h"
#include "game/ship.h"
#include "game/planet.h"
#include "game/mission.h"

static void
writeFCodesFor(GObject& obj, JsonWriter& w)
{
    if (obj.isPlayed()) {
        const GFCodeList& list = GFCodeList::getInstance();
        for (GFCodeList::iterator i = list.begin(); i != list.end(); ++i) {
            if (i->worksOn(obj)) {
                w.startObject();
                w.writeString("fc", i->getFCode());
                w.writeString("desc", i->getDescription());
                w.endObject();
            }
        }
    }
}

void
ServerShipFCWriter::write(JsonWriter& w)
{
    w.startArray(format("shipfc%d", sid));
    writeFCodesFor(getDisplayedTurn().getCurrentUniverse().getShip(sid), w);
    w.endArray();
}

void
ServerPlanetFCWriter::write(JsonWriter& w)
{
    w.startArray(format("planetfc%d", pid));
    writeFCodesFor(getDisplayedTurn().getCurrentUniverse().getPlanet(pid), w);
    w.endArray();
}

void
ServerShipMissionWriter::write(JsonWriter& w)
{
    /* The AJAX cannot yet handle all mission conditions, so we handle them serverside.
       The AJAX also formats mission names itself, so we don't send names. */
    w.startArray(format("shipmsn%d", sid));
    GShip& sh = getDisplayedTurn().getCurrentUniverse().getShip(sid);
    if (sh.isPlayed()) {
        const GMissionList& ml = GMissionList::getInstance();
        for (GMissionList::iterator i = ml.begin(); i != ml.end(); ++i) {
            if (i->worksOn(sh)) {
                w.startObject();
                w.writeInt("id", i->getNumber());
                w.writeInt("iarg", i->getParameterType(GMission::InterceptParameter));
                w.writeInt("targ", i->getParameterType(GMission::TowParameter));
                if (i->getParameterType(GMission::InterceptParameter))
                    w.writeString("iname", i->getParameterName(GMission::InterceptParameter));
                if (i->getParameterType(GMission::TowParameter))
                    w.writeString("tname", i->getParameterName(GMission::TowParameter));
                w.endObject();
            }
        }
    }
    w.endArray();
}
